package sun.security.jgss;

import java.io.OutputStream;
import org.ietf.jgss.GSSException;
import sun.security.util.DerValue;
import java.io.InputStream;
import java.io.IOException;
import sun.security.util.DerOutputStream;
import sun.security.util.ObjectIdentifier;

public class GSSHeader
{
    private ObjectIdentifier mechOid;
    private byte[] mechOidBytes;
    private int mechTokenLength;
    public static final int TOKEN_ID = 96;
    
    public GSSHeader(ObjectIdentifier mechOid, int mechTokenLength) throws IOException {
        this.mechOid = null;
        this.mechOidBytes = null;
        this.mechTokenLength = 0;
        this.mechOid = mechOid;
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.putOID(mechOid);
        this.mechOidBytes = derOutputStream.toByteArray();
        this.mechTokenLength = mechTokenLength;
    }
    
    public GSSHeader(InputStream in) throws IOException, GSSException {
        this.mechOid = null;
        this.mechOidBytes = null;
        this.mechTokenLength = 0;
        if (in.read() != 96) {
            throw new GSSException(10, -1, "GSSHeader did not find the right tag");
        }
        int length = this.getLength(in);
        DerValue derValue = new DerValue(in);
        this.mechOidBytes = derValue.toByteArray();
        this.mechOid = derValue.getOID();
        this.mechTokenLength = length - this.mechOidBytes.length;
        return;
    }
    
    public ObjectIdentifier getOid() {
        return this.mechOid;
    }
    
    public int getMechTokenLength() {
        return this.mechTokenLength;
    }
    
    public int getLength() {
        return 1 + this.getLenFieldSize(this.mechOidBytes.length + this.mechTokenLength) + this.mechOidBytes.length;
    }
    
    public static int getMaxMechTokenSize(ObjectIdentifier objectIdentifier, int n) {
        int length = 0;
        try {
            DerOutputStream derOutputStream = new DerOutputStream();
            derOutputStream.putOID(objectIdentifier);
            length = derOutputStream.toByteArray().length;
        }
        catch (IOException ex) {}
        n -= 1 + length;
        n -= 5;
        return n;
    }
    
    private int getLenFieldSize(int n) {
        int n2;
        if (n < 128) {
            n2 = 1;
        }
        else if (n < 256) {
            n2 = 2;
        }
        else if (n < 65536) {
            n2 = 3;
        }
        else if (n < 16777216) {
            n2 = 4;
        }
        else {
            n2 = 5;
        }
        return n2;
    }
    
    public int encode(OutputStream outputStream) throws IOException {
        int n = 1 + this.mechOidBytes.length;
        outputStream.write(96);
        int n2 = n + this.putLength(this.mechOidBytes.length + this.mechTokenLength, outputStream);
        outputStream.write(this.mechOidBytes);
        return n2;
    }
    
    private int getLength(InputStream inputStream) throws IOException {
        return this.getLength(inputStream.read(), inputStream);
    }
    
    private int getLength(int n, InputStream inputStream) throws IOException {
        if ((n & 0x80) == 0x0) {
        }
        else {
            int i = n & 0x7F;
            if (i == 0) {
                return -1;
            }
            if (i < 0 || i > 4) {
                throw new IOException("DerInputStream.getLength(): lengthTag=" + i + ", " + ((i < 0) ? "incorrect DER encoding." : "too big."));
            }
            n = 0;
            while (i > 0) {
                n = (n << 8) + (0xFF & inputStream.read());
                --i;
            }
            if (n < 0) {
                throw new IOException("Invalid length bytes");
            }
        }
        return n;
    }
    
    private int putLength(int n, OutputStream outputStream) throws IOException {
        int n2;
        if (n < 128) {
            outputStream.write((int)(byte)n);
            n2 = 1;
        }
        else if (n < 256) {
            outputStream.write(-127);
            outputStream.write((int)(byte)n);
            n2 = 2;
        }
        else if (n < 65536) {
            outputStream.write(-126);
            outputStream.write((int)(byte)(n >> 8));
            outputStream.write((int)(byte)n);
            n2 = 3;
        }
        else if (n < 16777216) {
            outputStream.write(-125);
            outputStream.write((int)(byte)(n >> 16));
            outputStream.write((int)(byte)(n >> 8));
            outputStream.write((int)(byte)n);
            n2 = 4;
        }
        else {
            outputStream.write(-124);
            outputStream.write((int)(byte)(n >> 24));
            outputStream.write((int)(byte)(n >> 16));
            outputStream.write((int)(byte)(n >> 8));
            outputStream.write((int)(byte)n);
            n2 = 5;
        }
        return n2;
    }
    
    private void debug(String s) {
        System.err.print(s);
    }
    
    private String getHexBytes(byte[] array, int n) throws IOException {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < n; ++i) {
            int j = array[i] >> 4 & 0xF;
            int k = array[i] & 0xF;
            sb.append(Integer.toHexString(j));
            sb.append(Integer.toHexString(k));
            sb.append(' ');
        }
        return sb.toString();
    }
}
